-----------------------------------------------------------------------------
--
-- data
--   \damageDealt
--     \<damageDealerName>
--       -count, amount, maximumAmount, maximumAmountAttack
--       \damageTypeDetails
--         \<damageTypeLocal>
--			 -count, amount
--       \attackDetails
--         \<attackName>
--            -count, amount, maximumAmount, damageType
--            \hitDetails
--			    \<attackType> -- one of Hit, Critical Hit, Tick, Miss, Dodge, Resist, etc.
--                -count, amount
--   \damageTaken
--     \<damageTakerName>
--       -count, amount, maximumAmount, maximumAmountAttac
--       \damageTypeDetails
--         \<damageTypeLocal>
--			 -count, amount
--       \attackDetails
--         \<attackName>
--            -count, amount, maximumAmount, damageType
--            \hitDetails
--  			\<attackType>
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDamage")
L:RegisterTranslations("enUS", function() return {
	["Damage"] = true,
	["Damage Options."] = true,
	["Merge Pet Damage"] = true,
	["Merges damage dealt by pets with their owners data."] = true,

	["Damage Dealers"] = true,
	["Damage Takers"] = true,
	["Damage Dealt"] = true,
	["Fight Damage Dealt"] = true,
	["Damage Dealt Details"] = true,
	["Damage Type Dealt Details"] = true,
	["Damage Dealt Hit Details"] = true,
	["DPS"] = true,
	
	["Highest Hits"] = true,

	["Damage Taken"] = true,
	["Damage Type Taken Details"] = true,
	["Damage Taken Details"] = true,
	["Damage Taken Hit Details"] = true,
	["DPS"] = true,
	["Fight Damage Taken"] = true,

	["Hit"] = true,
	["Critical Hit"] = true,
	["Tick"] = true,
	["Partial Absorb"] = true,
	["Partial Block"] = true,
	["Partial Resist"] = true,
	["Melee"] = true,
	
	["Pet: "] = true,

	["Damage Type"] = true,
	["Colors"] = true,
	["Color options"] = true,

	["Damage Type Data"] = true,
	["Information regarding damage types taken: Fire, Frost, etc."] = true,
	["Attack Data"] = true,
	["Information about each attack used."] = true,
	["Hit Data"] = true,
	["Detailed information about each attack used."] = true,
	["Max Data"] = true,
	["Gather maximum hit data."] = true,
	
	["FILTER_NONE"]	= "Gather no data.",
	["FILTER_SELF"] = "Gather self data only.",
	["FILTER_ALL"]	= "Gather all data.",
} end)

-----------------------------------------------------------------------------

local AssessmentDamage = Assessment:NewModule("Damage", "AceEvent-2.0", "Parser-3.0")
AssessmentDamage.db = Assessment:AcquireDBNamespace("Damage")
Assessment:RegisterDefaults("Damage", "profile", {
	enable						= true,
	mergePetWithOwner			= true,
	gatherDamageTypeDataFilter	= "ALL",
	gatherAttackDataFilter		= "ALL",
	gatherHitDataFilter			= "ALL",
	gatherMaximumDataFilter		= "ALL",
	colors =
	{
		physical	= { 1, 0.75, 0.46 },
		holy		= { 1, 1, 0.5 },
		fire		= { 1, 0.3, 0 },
		nature		= { 0.5, 1, 0.2 },
		frost		= { 0.4, 0.6, 0.9 },
		shadow		= { 0.7, 0.4, 1 },
		arcane		= { 0.8, 0.8, 1 },
		unknown		= { 0, 0, 0 },
	}
})

local profile
local profileColors
local gatherDamageTypeMethod
local gatherAttackDataMethod
local gatherHitDataMethod
local gatherMaximumDataMethod

local emptyMethod = Assessment.EmptyMethod

-----------------------------------------------------------------------------

Assessment.TagData.damagedealt =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local damageDealt = fightData.damageDealt
				if not damageDealt then return 0 end
				
				if not damageDealt[data.key] then return 0 end
				return damageDealt[data.key].amount
			 end,
	data = "damage",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.maximumdamagedealt =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local damageDealt = fightData.damageDealt
				if not damageDealt then return 0 end
				
				if not damageDealt[data.key] then return 0 end
				return damageDealt[data.key].maximumAmount or 0
			 end,
	data = "damage",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.maximumdamagedealtattack =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return end
				
				local damageDealt = fightData.damageDealt
				if not damageDealt then return end
				
				if not damageDealt[data.key] then return end
				return damageDealt[data.key].maximumAmountAttack
			 end,
	data = "damage",
}

Assessment.TagData.damagetaken =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local damageTaken = fightData.damageTaken
				if not damageTaken then return 0 end
				
				if not damageTaken[data.key] then return 0 end
				return damageTaken[data.key].amount
			 end,
	data = "damage",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.maximumdamagetaken =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local damageTaken = fightData.damageTaken
				if not damageTaken then return 0 end
				
				if not damageTaken[data.key] then return 0 end
				return damageTaken[data.key].maximumAmount or 0
			 end,
	data = "damage",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.maximumdamagetakenattack =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return end
				
				local damageTaken = fightData.damageTaken
				if not damageTaken then return end
				
				if not damageTaken[data.key] then return end
				return damageTaken[data.key].maximumAmountAttack
			 end,
	data = "damage",
}

Assessment.TagData.maximumamountattack =
{
	tableData = "value.maximumAmountAttack",
	numeric = true
}

Assessment.TagData.absorbamount =
{
	tableData = "value.absorbAmount",
	numeric = true
}

Assessment.TagData.blockamount =
{
	tableData = "value.blockAmount",
	numeric = true
}

Assessment.TagData.misscount =
{
	tableData = "value.count",
	numeric = true
}

Assessment.TagData.resistamount =
{
	tableData = "value.resistAmount",
	numeric = true
}

-----------------------------------------------------------------------------

Assessment.ListType.DamageDealers =
{
	method = function(fightData)
				if not fightData or not fightData.damageDealt then return emptyMethod end
				return pairs(fightData.damageDealt)
			 end,
	data = "damage",
	description = L["Damage Dealers"],
}

Assessment.ListType.DamageTakers =
{
	method = function(fightData)
				if not fightData or not fightData.damageTaken then return emptyMethod end
				return pairs(fightData.damageTaken)
			 end,
	data = "damage",
	description = L["Damage Takers"],
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.damageDealt = "damage"
Assessment.ListType.EnumerateTable.updateData.damageTaken = "damage"

-----------------------------------------------------------------------------

local playerName = UnitName("player")
local gatherFilterMethods =
{
	NONE	= function() return false end,
	SELF	= function(name) return name == playerName end,
	ALL		= function() return true end,
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.DamageType =
{
	titleTag			= "[windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "[windowParameterList],damageTypeDetails",
	barValueTag			= "amount", 
	filterTag			= "1",
	sortKeyTag			= "barValue",
	fightIndex			= -1,
	text				= { { }, { tag= "[barValue:short]" } },
	description			= L["Damage Type"],
	colorTypeList		= { "DamageType", count = 1 },
	favorite			= false,
	selectable			= false,
}

Assessment.DefaultDisplaySets.DamageDealtHitDetails =
{
	titleTag			= "[windowParameter(1)]: [windowParameter(2)]",
	listType			= "EnumerateTable", 
	listDataTag			= "damageDealt,[windowParameter(1)],attackDetails,[windowParameter(2)],hitDetails",
	barValueTag			= "amount",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { { tag = "[rank]. [key] ([count])" }, { tag = "[barValue:short]" } },
	description			= L["Damage Dealt Hit Details"],
	selectable			= false,
--	colorTypeList		= { "Custom" },
}

Assessment.DefaultDisplaySets.DamageDealtDetails =
{
	titleTag			= "[(fightIndex ~= 0):\"Fight \"]Damage Dealt: [windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "damageDealt,[windowParameter(1)],attackDetails",
	barValueTag			= "amount",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { { tag = "[rank]. [key] ([count])" }, { tag = "[barValue:short]" }, { tag = "Average: [(barValue/count):format(\"%.1f\")][maximumAmount:prefix(\", Maximum: \")]", visible = false }, count = 3 },
	description			= L["Damage Dealt Details"],
	selectable			= false,
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Damage Dealt Hit Details"],
	onClickData			= "[windowParameter(1)],[key]",
	colorTypeList		= { "SpellDamageType", count = 1 },
}

Assessment.DefaultDisplaySets.DamageTypeDealtDetails =
{
	titleTag			= "[(fightIndex ~= 0):\"Fight \"]Damage Type Dealt: [windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "damageDealt,[windowParameter(1)],damageTypeDetails",
	barValueTag			= "amount",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { { tag = "[rank]. [key] ([count])" }, { tag = "[barValue:short]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3 },
	description			= L["Damage Type Dealt Details"],
	selectable			= false,
	colorTypeList		= { "DamageType", count = 1 },
}

Assessment.DefaultDisplaySets.DamageDealt =
{
	titleTag			= "Damage Dealt: [totalBarValue:short]",
	listType			= "GroupMembers", 
	barValueTag			= "damageDealt", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 0,
	text				= { { }, { tag= "[barValue:short]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3 },
	description			= L["Damage Dealt"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= string.format("[select(isControlKeyDown, %q, %q)]", L["Damage Type Dealt Details"], L["Damage Dealt Details"]),
	onClickData			= "[key]",
	segmentEnable		= false,
	segmentDisplaySet	= L["Damage Type"],
	segmentData			= "damageDealt,[key]",
	colorTypeList		= { "Class", count = 1 },
}

Assessment.DefaultDisplaySets.FightDamageDealt =
{
	titleTag			= "Fight Damage Dealt: [totalBarValue:short]",
	listType			= "GroupMembers", 
	barValueTag			= "damageDealt", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 1,
	text				= { { }, { tag= "[barValue:short]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3 },
	description			= L["Fight Damage Dealt"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= string.format("[select(isControlKeyDown, %q, %q)]", L["Damage Type Dealt Details"], L["Damage Dealt Details"]),
	onClickData			= "[key]",
	segmentEnable		= false,
	segmentDisplaySet	= L["Damage Type"],
	segmentData			= "damageDealt,[key]",
	colorTypeList		= { "Class", count = 1 },
}

Assessment.DefaultDisplaySets.HighestHits =
{
	titleTag			= "Highest Hits",
	listType			= "GroupMembers", 
	barValueTag			= "maximumDamageDealt", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 0,
	text				= { { tag = "[rank]. [key]: [maximumDamageDealtAttack]" }, { tag= "[barValue:short]" } },
	description			= L["Highest Hits"],
	colorTypeList		= { "Class", count = 1 },
	favorite			= false,
}

Assessment.DefaultDisplaySets.DPS =
{
	titleTag			= "DPS: [combatDuration:formatDuration]",
	listType			= "GroupMembers", 
	barValueTag			= "damageDealt / maximum(combatDuration, 1)", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 1,
	text				= { { }, { tag= "[barValue:format(\"%.1f\")]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3  },
	description			= L["DPS"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= string.format("[select(isControlKeyDown, %q, %q)]", L["Damage Type Dealt Details"], L["Damage Dealt Details"]),
	onClickData			= "[key]",
	favorite			= false,
	segmentEnable		= false,
	segmentDisplaySet	= L["Damage Type"],
	segmentData			= "damageDealt,[key]",
	colorTypeList		= { "Class", count = 1 },
}

Assessment.DefaultDisplaySets.DamageTakentHitDetails =
{
	titleTag			= "[windowParameter(1)]: [windowParameter(2)]",
	listType			= "EnumerateTable", 
	listDataTag			= "damageTaken,[windowParameter(1)],attackDetails,[windowParameter(2)],hitDetails",
	barValueTag			= "amount",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { { tag = "[rank]. [key] ([count])" }, { tag = "[barValue:short]" } },
	description			= L["Damage Taken Hit Details"],
	selectable			= false,
--	colorTypeList		= { "Spell" },
}

Assessment.DefaultDisplaySets.DamageTypeTakenDetails =
{
	titleTag			= "[(fightIndex ~= 0):\"Fight \"]Damage Type Taken: [windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "damageTaken,[windowParameter(1)],damageTypeDetails",
	barValueTag			= "amount",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { { tag = "[rank]. [key] ([count])" }, { tag = "[barValue:short]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3 },
	description			= L["Damage Type Taken Details"],
	selectable			= false,
	colorTypeList		= { "DamageType", count = 1 },
}

Assessment.DefaultDisplaySets.DamageTakenDetails =
{
	titleTag			= "[(fightIndex ~= 0):\"Fight \"]Damage Taken: [windowParameter(1)]",
	listType			= "EnumerateTable", 
	listDataTag			= "damageTaken,[windowParameter(1)],attackDetails",
	barValueTag			= "amount",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { { tag = "[rank]. [key] ([count])" }, { tag = "[barValue:short]" }, { tag = "Average: [(barValue/count):format(\"%.1f\")][maximumAmount:prefix(\", Maximum: \")]", visible = false }, count = 3 },
	description			= L["Damage Taken Details"],
	selectable			= false,
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Damage Taken Hit Details"],
	onClickData			= "[windowParameter(1)],[key]",
	colorTypeList		= { "SpellDamageType", count = 1 },
}

Assessment.DefaultDisplaySets.DamageTaken =
{
	titleTag			= "Damage Taken: [totalBarValue:short]",
	listType			= "GroupMembers", 
	barValueTag			= "damageTaken", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 0,
	text				= { { }, { tag= "[barValue:short]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3 },
	description			= L["Damage Taken"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= string.format("[select(isControlKeyDown, %q, %q)]", L["Damage Type Taken Details"], L["Damage Taken Details"]),
	onClickData			= "[key]",
	segmentEnable		= true,
	segmentDisplaySet	= L["Damage Type"],
	segmentData			= "damageTaken,[key]",
	colorTypeList		= { "Class", count = 1 },
}

Assessment.DefaultDisplaySets.FightDamageTaken =
{
	titleTag			= "Fight Damage Taken: [totalBarValue:short]",
	listType			= "GroupMembers", 
	barValueTag			= "damageTaken", 
	filterTag			= "barValue ~= 0",
	fightIndex			= 1,
	text				= { { }, { tag= "[barValue:short]" }, { tag = "[(barValue*100/totalBarValue):format(\"%.1f%%\")]", visible = false }, count = 3 },
	description			= L["Fight Damage Taken"],
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= string.format("[select(isControlKeyDown, %q, %q)]", L["Damage Type Taken Details"], L["Damage Taken Details"]),
	onClickData			= "[key]",
	segmentEnable		= true,
	segmentDisplaySet	= L["Damage Type"],
	segmentData			= "damageTaken,[key]",
	colorTypeList		= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

local function getColor(passValue)
	return unpack(profileColors[passValue])
end

local function setColor(passValue,r, g, b)
	profileColors[passValue] = { r, g, b }
	Assessment:RedrawAll()
end

Assessment.Options.args.data.args.damage =
{
	type = "group",
	name = L["Damage"],
	desc = L["Damage Options."],
	args =
	{
		gatherDamageType =
		{
			type = "text",
			name = L["Damage Type Data"],
			desc = L["Information regarding damage types taken: Fire, Frost, etc."],
			get = function() return profile.gatherDamageTypeDataFilter end,
			set = function(v) profile.gatherDamageTypeDataFilter = v AssessmentDamage:OnInitialize() end,
			validate = { NONE = L["FILTER_NONE"], SELF = L["FILTER_SELF"], ALL = L["FILTER_ALL"] }
		},
		gatherAttackData =
		{
			type = "text",
			name = L["Attack Data"],
			desc = L["Information about each attack used."],
			get = function() return profile.gatherAttackDataFilter end,
			set = function(v) profile.gatherAttackDataFilter = v AssessmentDamage:OnInitialize() end,
			validate = { NONE = L["FILTER_NONE"], SELF = L["FILTER_SELF"], ALL = L["FILTER_ALL"] }
		},
		gatherHitData =
		{
			type = "text",
			name = L["Hit Data"],
			desc = L["Detailed information about each attack used."],
			get = function() return profile.gatherHitDataFilter end,
			set = function(v) profile.gatherHitDataFilter = v AssessmentDamage:OnInitialize() end,
			validate = { NONE = L["FILTER_NONE"], SELF = L["FILTER_SELF"], ALL = L["FILTER_ALL"] }
		},
		gatherHitData =
		{
			type = "text",
			name = L["Max Data"],
			desc = L["Gather maximum hit data."],
			get = function() return profile.gatherMaximumDataFilter end,
			set = function(v) profile.gatherMaximumDataFilter = v AssessmentDamage:OnInitialize() end,
			validate = { NONE = L["FILTER_NONE"], SELF = L["FILTER_SELF"], ALL = L["FILTER_ALL"] }
		},
		mergePetWithOwner =
		{
			type = "toggle",
			name = L["Merge Pet Damage"],
			desc = L["Merges damage dealt by pets with their owners data."],
			get = function() return profile.mergePetWithOwner end,
			set = function(v) profile.mergePetWithOwner = v end,
		},
		color =
		{
			type = "group",
			name = L["Colors"],
			desc = L["Color options"],
			args =
			{
				-- Filled in below
			},
		},
	}
}

-----------------------------------------------------------------------------

local localDamageTypeToColorName = 
{ 
	[SPELL_SCHOOL0_CAP] = "physical",
	[SPELL_SCHOOL1_CAP] = "holy",
	[SPELL_SCHOOL2_CAP] = "fire",
	[SPELL_SCHOOL3_CAP] = "nature",
	[SPELL_SCHOOL4_CAP] = "frost",
	[SPELL_SCHOOL5_CAP] = "shadow",
	[SPELL_SCHOOL6_CAP] = "arcane",
	[UNKNOWN]			= "unknown"
}

for k,v in pairs(localDamageTypeToColorName) do
	Assessment.Options.args.data.args.damage.args.color.args[v] =
	{
		type = "color",
		name = k,
		desc = k,
		get = getColor,
		set = setColor,
		passValue = v
	}
end

-----------------------------------------------------------------------------

Assessment.ColorType.DamageType = 
{
	method = function(fightData, data)
				local colorName = localDamageTypeToColorName[data.key]
				if not colorName then return nil end
				return profileColors[colorName]
			 end,
	description = L["Damage Type"]
}

Assessment.ColorType.SpellDamageType = 
{
	method = function(fightData, data)
				local damageType = data.value.damageType
				if not damageType then return end
				
				return profileColors[damageType]
			 end,
	description = L["Damage Type"]
}

-----------------------------------------------------------------------------

Assessment.DefaultTags.DPS =
{
	tag			= "[(damageDealt / maximum(combatDuration, 1)):format(\"%.1f\")]",
	fightIndex	= 1,
	description	= L["DPS"],
}

-----------------------------------------------------------------------------

function AssessmentDamage:OnInitialize()
	profile = self.db.profile
	profileColors = profile.colors
	gatherDamageTypeDataMethod = gatherFilterMethods[profile.gatherDamageTypeDataFilter]
	gatherAttackDataMethod = gatherFilterMethods[profile.gatherAttackDataFilter]
	gatherHitDataMethod = gatherFilterMethods[profile.gatherHitDataFilter]
	gatherMaximumDataMethod = gatherFilterMethods[profile.gatherMaximumDataFilter]
end

function AssessmentDamage:OnEnable()
	self:RegisterParserEvent({ eventType = "Damage" }, "OnParserDamageEvent")
	self:RegisterParserEvent({ eventType = "Miss" }, "OnParserMissEvent")
end

function AssessmentDamage:OnDisable()
end

-----------------------------------------------------------------------------

local function AccumulateData(data, eventData)
	data.count = data.count + 1
	data.amount = data.amount + eventData.amount
end

local petToOwnerTable = { pet = "player" }
for i=1,4 do petToOwnerTable["partypet"..i] = "party"..i end
for i=1,40 do petToOwnerTable["raidpet"..i] = "raid"..i end

-----------------------------------------------------------------------------

function AssessmentDamage:NewDataTable()
	local data = self:NewTable()
	data.count = 0
	data.amount = 0
	return data
end

-----------------------------------------------------------------------------

function AssessmentDamage:UpdateDamageData(dataTable, member, eventData, attackType, abilityName)
	local data = dataTable[member]
	if not data then
		data = self:NewDataTable()
		data.attackDetails = self:NewTable()
		data.damageTypeDetails = self:NewTable()
		dataTable[member] = data
	end
	AccumulateData(data, eventData)
	
	local gatherMaximum = gatherMaximumDataMethod(member)
	if gatherMaximum then
		local amount = eventData.amount
		if not data.maximumAmount then
			data.maximumAmount = amount
			data.maximumAmountAttack = abilityName
		elseif amount > data.maximumAmount then
			data.maximumAmount = amount
			data.maximumAmountAttack = abilityName
		end
	end

	if gatherDamageTypeDataMethod(member) then
		local damageTypeDetailsParent = data.damageTypeDetails
		local damageTypeDetails = damageTypeDetailsParent[eventData.damageTypeLocal]
		if not damageTypeDetails then
			damageTypeDetails = self:NewDataTable()
			damageTypeDetailsParent[eventData.damageTypeLocal] = damageTypeDetails
		end
		AccumulateData(damageTypeDetails, eventData)
	end

	if gatherAttackDataMethod(member) then	
		local attackDetailsParent = data.attackDetails
		local attackDetails = attackDetailsParent[abilityName]
		if not attackDetails then
			attackDetails = self:NewDataTable()
			attackDetails.hitDetails = self:NewTable()
			attackDetailsParent[abilityName] = attackDetails
		end
		attackDetails.damageType = localDamageTypeToColorName[eventData.damageTypeLocal]
		AccumulateData(attackDetails, eventData)

		if gatherMaximum then
			local amount = eventData.amount
			if not attackDetails.maximumAmount then
				attackDetails.maximumAmount = amount
				attackDetails.maximumAmountAttack = abilityName
			elseif amount > attackDetails.maximumAmount then
				attackDetails.maximumAmount = amount
				attackDetails.maximumAmountAttack = abilityName
			end
		end
		
		if gatherHitDataMethod(member) then
			local hitDetails = attackDetails.hitDetails
			local attackTypeDetails = hitDetails[attackType]
			if not attackTypeDetails then
				attackTypeDetails = self:NewDataTable()
				hitDetails[attackType] = attackTypeDetails
			end
			AccumulateData(attackTypeDetails, eventData)
			
			if eventData.absorbAmount ~= 0 then
				local type = L["Partial Absorb"]
				local attackTypeDetails = hitDetails[type]
				if not attackTypeDetails then
					attackTypeDetails = self:NewDataTable()
					hitDetails[type] = attackTypeDetails
				end
				attackTypeDetails.count = attackTypeDetails.count + 1
				attackTypeDetails.amount = attackTypeDetails.amount + eventData.absorbAmount
			end

			if eventData.blockAmount ~= 0 then
				local type = L["Partial Block"]
				local attackTypeDetails = hitDetails[type]
				if not attackTypeDetails then
					attackTypeDetails = self:NewDataTable()
					hitDetails[type] = attackTypeDetails
				end
				attackTypeDetails.count = attackTypeDetails.count + 1
				attackTypeDetails.amount = attackTypeDetails.amount + eventData.blockAmount
			end

			if eventData.resistAmount ~= 0 then
				local type = L["Partial Resist"]
				local attackTypeDetails = hitDetails[type]
				if not attackTypeDetails then
					attackTypeDetails = self:NewDataTable()
					hitDetails[type] = attackTypeDetails
				end
				attackTypeDetails.count = attackTypeDetails.count + 1
				attackTypeDetails.amount = attackTypeDetails.amount + eventData.resistAmount
			end
		end
	end
end

-----------------------------------------------------------------------------

function AssessmentDamage:OnParserDamageEvent(eventData)
	local attackType
	if eventData.isDoT then
		attackType = L["Tick"]
	elseif eventData.isCrit then
		attackType = L["Critical Hit"]
	else
		attackType = L["Hit"]
	end
	
	local abilityName = eventData.abilityName or L["Melee"]

	local sourceName = eventData.sourceName
	if profile.mergePetWithOwner then
		local sourceOwnerID = petToOwnerTable[eventData.sourceID]
		if sourceOwnerID then
			abilityName = L["Pet: "]..abilityName
			sourceName = UnitName(sourceOwnerID)
		end
	end
	
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.damage = GetTime()
		
		local damageDealt = data.damageDealt
		if not damageDealt then
			damageDealt = self:NewTable()
			data.damageDealt = damageDealt
		end

		local damageTaken = data.damageTaken
		if not damageTaken then
			damageTaken = self:NewTable()
			data.damageTaken = damageTaken
		end

		self:UpdateDamageData(damageDealt, sourceName, eventData, attackType, abilityName)
		self:UpdateDamageData(damageTaken, eventData.recipientName, eventData, attackType, abilityName)
	end
end

-----------------------------------------------------------------------------

function AssessmentDamage:UpdateMissData(dataTable, member, eventData, abilityName)
	local data = dataTable[member]
	if not data then
		data = self:NewDataTable()
		data.attackDetails = self:NewTable()
		data.damageTypeDetails = self:NewTable()
		dataTable[member] = data
	end
	data.count = data.count + 1
	
	if gatherAttackDataMethod(member) then
		local attackDetailsParent = data.attackDetails
		local attackDetails = attackDetailsParent[abilityName]
		if not attackDetails then
			attackDetails = self:NewDataTable()
			attackDetails.hitDetails = self:NewTable()
			attackDetailsParent[abilityName] = attackDetails
		end
		attackDetails.count = attackDetails.count + 1
		
		if gatherHitDataMethod(member) then		
			local hitDetails = attackDetails.hitDetails
			local attackTypeDetails = hitDetails[eventData.missTypeLocal]
			if not attackTypeDetails then
				attackTypeDetails = self:NewDataTable()
				hitDetails[eventData.missTypeLocal] = attackTypeDetails
			end
			attackTypeDetails.count = attackTypeDetails.count + 1
		end
	end
end

-----------------------------------------------------------------------------

function AssessmentDamage:OnParserMissEvent(eventData)
	local abilityName = eventData.abilityName or L["Melee"]
	
	local sourceName = eventData.sourceName
	if profile.mergePetWithOwner then
		local sourceOwnerID = petToOwnerTable[eventData.sourceID]
		if sourceOwnerID then
			abilityName = L["Pet: "]..abilityName
			sourceName = UnitName(sourceOwnerID)
		end
	end
	
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.damage = GetTime()
		
		local damageDealt = data.damageDealt
		if not damageDealt then
			damageDealt = self:NewTable()
			data.damageDealt = damageDealt
		end

		local damageTaken = data.damageTaken
		if not damageTaken then
			damageTaken = self:NewTable()
			data.damageTaken = damageTaken
		end

		self:UpdateMissData(damageDealt, sourceName, eventData, abilityName)
		self:UpdateMissData(damageTaken, eventData.recipientName, eventData, abilityName)
	end
end

-----------------------------------------------------------------------------
